@@ -1,6 +1,6 @@
 from dataclasses import dataclass
 
-from decouple import config
+from nova.config import NOVA_AUTH0_AUDIENCE, NOVA_AUTH0_CLIENT_ID, NOVA_AUTH0_DOMAIN
 
 
 @dataclass
@@ -15,9 +15,7 @@ class Auth0Config:
     def from_env(cls) -> "Auth0Config":
         """Create Auth0Config from environment variables"""
         return cls(
-            domain=config("NOVA_AUTH0_DOMAIN", default="auth.portal.wandelbots.io"),
-            client_id=config("NOVA_AUTH0_CLIENT_ID", default="J7WJUi38xVQdJAEBNRT9Xw1b0fXDb4J2"),
-            audience=config("NOVA_AUTH0_AUDIENCE", default="nova-api"),
+            domain=NOVA_AUTH0_DOMAIN, client_id=NOVA_AUTH0_CLIENT_ID, audience=NOVA_AUTH0_AUDIENCE
         )
 
     def is_complete(self) -> bool:
