@@ -300,6 +300,7 @@ class AbstractRobot(Device):
         tcp: str,
         actions: list[Action],
         movement_controller: MovementController | None,
+        start_on_io: api.models.start_on_io.StartOnIO | None = None,
     ) -> AsyncIterable[MovementResponse]:
         """Execute a planned motion
 
@@ -308,6 +309,7 @@ class AbstractRobot(Device):
             tcp (str): The id of the tool center point (TCP)
             actions (list[Action] | Action | None): The actions to be executed. Defaults to None.
             movement_controller (MovementController): The movement controller to be used. Defaults to move_forward
+            start_on_io (StartOnIO | None): The start on IO. If none, does not wait for IO. Defaults to None.
         """
 
     async def stream_execute(
@@ -316,6 +318,7 @@ class AbstractRobot(Device):
         tcp: str,
         actions: list[Action] | Action,
         movement_controller: MovementController | None = None,
+        start_on_io: api.models.StartOnIO | None = None,
     ) -> AsyncIterable[MotionState]:
         """Execute a planned motion
 
@@ -324,6 +327,7 @@ class AbstractRobot(Device):
             tcp (str): The id of the tool center point (TCP)
             actions (list[Action] | Action | None): The actions to be executed. Defaults to None.
             movement_controller (MovementController): The movement controller to be used. Defaults to move_forward
+            start_on_io (StartOnIO | None): The start on IO. If none, does not wait for IO. Defaults to None.
         """
         if not isinstance(actions, list):
             actions = [actions]
@@ -347,7 +351,11 @@ class AbstractRobot(Device):
             assert False, f"Unexpected movement response: {movement_response}"
 
         execute_response_stream = self._execute(
-            joint_trajectory, tcp, actions, movement_controller=movement_controller
+            joint_trajectory,
+            tcp,
+            actions,
+            movement_controller=movement_controller,
+            start_on_io=start_on_io,
         )
         motion_states = (
             stream.iterate(execute_response_stream)
@@ -365,6 +373,7 @@ class AbstractRobot(Device):
         tcp: str,
         actions: list[Action] | Action,
         movement_controller: MovementController | None = None,
+        start_on_io: api.models.StartOnIO | None = None,
     ) -> None:
         """Execute a planned motion
 
@@ -373,9 +382,14 @@ class AbstractRobot(Device):
             tcp (str): The id of the tool center point (TCP)
             actions (list[Action] | Action): The actions to be executed.
             movement_controller (MovementController): The movement controller to be used. Defaults to move_forward
+            start_on_io (StartOnIO | None): The start on IO. If none, does not wait for IO. Defaults to None.
         """
         async for _ in self.stream_execute(
-            joint_trajectory, tcp, actions, movement_controller=movement_controller
+            joint_trajectory,
+            tcp,
+            actions,
+            movement_controller=movement_controller,
+            start_on_io=start_on_io,
         ):
             pass
 
