@@ -9,11 +9,18 @@ from typing import AsyncGenerator, TypeVar
 from urllib.parse import quote as original_quote
 
 import wandelbots_api_client as wb
-from decouple import config
+import wandelbots_api_client.v2 as v2
 
 from nova.auth.auth_config import Auth0Config
 from nova.auth.authorization import Auth0DeviceAuthorization
 from nova.cell.robot_cell import ConfigurablePeriphery, Device
+from nova.config import (  # add to the module for backward compatibility
+    INTERNAL_CLUSTER_NOVA_API,  # noqa: F401
+    NOVA_ACCESS_TOKEN,
+    NOVA_API,
+    NOVA_PASSWORD,
+    NOVA_USERNAME,
+)
 from nova.core import logger
 from nova.core.env_handler import set_key
 from nova.core.exceptions import LoadPlanFailed, PlanTrajectoryFailed
@@ -30,8 +37,6 @@ def _custom_quote_for_ios(param, safe=""):
 
 T = TypeVar("T")
 
-INTERNAL_CLUSTER_NOVA_API = "http://api-gateway.wandelbots.svc.cluster.local:8080"
-
 
 class ComparisonType(Enum):
     COMPARISON_TYPE_EQUAL = "COMPARISON_TYPE_EQUAL"
@@ -96,6 +101,9 @@ def intercept(api_instance: T, gateway: "ApiGateway") -> T:
 
 
 class ApiGateway:
+    _api_client: wb.ApiClient
+    _api_v2_client: v2.ApiClient
+
     def __init__(
         self,
         *,
@@ -107,17 +115,10 @@ class ApiGateway:
         verify_ssl: bool = True,
         auth0_config: Auth0Config | None = None,
     ):
-        if host is None:
-            host = config("NOVA_API", default=INTERNAL_CLUSTER_NOVA_API)
-
-        if username is None:
-            username = config("NOVA_USERNAME", default=None)
-
-        if password is None:
-            password = config("NOVA_PASSWORD", default=None)
-
-        if access_token is None:
-            access_token = config("NOVA_ACCESS_TOKEN", default=None)
+        host = host or NOVA_API
+        access_token = access_token or NOVA_ACCESS_TOKEN
+        username = username or NOVA_USERNAME
+        password = password or NOVA_PASSWORD
 
         self._version = version
         self._verify_ssl = verify_ssl
@@ -136,15 +137,6 @@ class ApiGateway:
             self._auth0 = Auth0DeviceAuthorization(auth0_config=auth0_config)
 
         self._host = self._host_with_prefix(host=host)
-        stripped_host = self._host.rstrip("/")
-        api_client_config = wb.Configuration(
-            host=f"{stripped_host}/api/{version}",
-            username=username,
-            password=password,
-            access_token=access_token,
-        )
-        api_client_config.verify_ssl = verify_ssl
-
         self._access_token = access_token
         self._username = username
         self._password = password
@@ -167,14 +159,15 @@ class ApiGateway:
             ios_api_module.quote = _custom_quote_for_ios
 
         stripped_host = self._host.rstrip("/")
+
+        # init v1 api client
         api_client_config = wb.Configuration(
-            host=f"{stripped_host}/api/{self._version}",
+            host=f"{stripped_host}/api/v1",
             username=self._username,
             password=self._password,
             access_token=self._access_token,
         )
         api_client_config.verify_ssl = self._verify_ssl
-
         self._api_client = wb.ApiClient(configuration=api_client_config)
         self._api_client.user_agent = f"Wandelbots-Nova-Python-SDK/{pkg_version}"
 
@@ -209,6 +202,21 @@ class ApiGateway:
         self.motion_group_jogging_api = intercept(
             wb.MotionGroupJoggingApi(api_client=self._api_client), self
         )
+        self.store_object_api = intercept(wb.StoreObjectApi(api_client=self._api_client), self)
+
+        # init v2 api client
+        api_v2_client_config = v2.Configuration(
+            host=f"{stripped_host}/api/v2",
+            username=self._username,
+            password=self._password,
+            access_token=self._access_token,
+        )
+        self._api_v2_client = v2.ApiClient(configuration=api_v2_client_config)
+        self._api_v2_client.user_agent = f"Wandelbots-Nova-Python-SDK/{pkg_version}"
+
+        self.virtual_robot_api_v2 = intercept(
+            v2.VirtualControllerApi(api_client=self._api_v2_client), self
+        )
 
         logger.debug(f"NOVA API client initialized with user agent {self._api_client.user_agent}")
 
