@@ -14,7 +14,7 @@ Vector3d(x=0.0, y=1.0, z=5.0)
 import wandelscript.antlrvisitor  # load Program.from_code # noqa: F401
 from wandelscript import builtins, motions
 from wandelscript.metamodel import Program, register_builtin_func
-from wandelscript.runner import ProgramRunner, create_wandelscript_program, run
+from wandelscript.runner import WandelscriptProgramRunner, run, run_wandelscript_program
 from wandelscript.runtime import Store
 from wandelscript.version import version
 
@@ -28,11 +28,11 @@ def analyze(code: str):
 __all__ = [
     "run",
     "Program",
-    "ProgramRunner",
+    "WandelscriptProgramRunner",
     "Store",
     "__version__",
     "register_builtin_func",
     "motions",
     "builtins",
-    "create_wandelscript_program",
+    "run_wandelscript_program",
 ]
