@@ -6,8 +6,7 @@ import nova
 from nova import Nova
 from nova.cell.robot_cell import RobotCell
 from nova.events import CycleDevice
-from nova.program import Program
-from nova.program import ProgramRunner as NovaProgramRunner
+from nova.program import ProgramRunner
 from nova.program.runner import ExecutionContext as NovaExecutionContext
 from wandelscript.datatypes import ElementType
 from wandelscript.ffi import ForeignFunction
@@ -16,25 +15,31 @@ from wandelscript.metamodel import Program as WandelscriptProgram
 from wandelscript.runtime import ExecutionContext
 
 
-class ProgramRunner(NovaProgramRunner):
+class WandelscriptProgramRunner(ProgramRunner):
     """Provides functionalities to manage a single program execution"""
 
     def __init__(
         self,
         program_id: str,
-        program: str,
-        args: dict[str, ElementType] | None,
+        code: str,
+        parameters: dict[str, ElementType] | None,
         robot_cell_override: RobotCell | None = None,
         default_robot: str | None = None,
         default_tcp: str | None = None,
         foreign_functions: dict[str, ForeignFunction] | None = None,
     ):
+        async def wandelscript_wrapper():
+            print(f"Running wandelscript program {program_id}...")
+
+        program = nova.program(id=program_id)(wandelscript_wrapper)
+
         super().__init__(
-            program_id=program_id,
-            args=args,  # type: ignore
+            program,
+            parameters=parameters,  # type: ignore
             robot_cell_override=robot_cell_override,
         )
-        self._program: str = program
+        self._program = wandelscript_wrapper
+        self._code: str = code
         self._default_robot: str | None = default_robot
         self._default_tcp: str | None = default_tcp
         self._foreign_functions: dict[str, ForeignFunction] = foreign_functions or {}
@@ -43,18 +48,18 @@ class ProgramRunner(NovaProgramRunner):
     async def _run(self, execution_context: NovaExecutionContext):
         # Try parsing the program and handle parsing error
         logger.info(f"Parse program {self.program_id}...")
-        logger.debug(self._program)
+        logger.debug(self._code)
 
         self._ws_execution_context = ws_execution_context = ExecutionContext(
             robot_cell=execution_context.robot_cell,
             stop_event=execution_context.stop_event,
             default_robot=self._default_robot,
             default_tcp=self._default_tcp,
-            run_args=self._args,
+            run_args=self._parameters,
             foreign_functions=self._foreign_functions,
         )
 
-        program = WandelscriptProgram.from_code(self._program)
+        program = WandelscriptProgram.from_code(self._code)
         # Execute Wandelscript
         await program(ws_execution_context)
         self.execution_context.motion_group_recordings = (
@@ -65,18 +70,19 @@ class ProgramRunner(NovaProgramRunner):
 
 def run(
     program_id: str,
-    program: str,
-    args: dict[str, ElementType] | None = None,
+    code: str,
+    parameters: dict[str, ElementType] | None = None,
     default_robot: str | None = None,
     default_tcp: str | None = None,
     foreign_functions: dict[str, ForeignFunction] | None = None,
     robot_cell_override: RobotCell | None = None,
-) -> ProgramRunner:
+) -> WandelscriptProgramRunner:
     """Helper function to create a ProgramRunner and start it synchronously
 
     Args:
-        program (str): Wandelscript code
-        args (dict[str, Any], optional): Store will be initialized with this dict. Defaults to ().
+        program_id (str): The unique identifier of the program.
+        code (str): Wandelscript code
+        parameters (dict[str, Any], optional): Store will be initialized with this dict. Defaults to ().
         default_robot (str): The default robot that is used when no robot is active
         default_tcp (str): The default TCP that is used when no TCP is explicitly selected for a motion
         foreign_functions (dict[str, ForeignFunction], optional): 3rd party functions that you can
@@ -87,10 +93,10 @@ def run(
         ProgramRunner: A new ProgramRunner object
 
     """
-    runner = ProgramRunner(
+    runner = WandelscriptProgramRunner(
         program_id=program_id,
-        program=program,
-        args=args,
+        code=code,
+        parameters=parameters,
         default_robot=default_robot,
         default_tcp=default_tcp,
         foreign_functions=foreign_functions,
@@ -100,37 +106,40 @@ def run(
     return runner
 
 
-def create_wandelscript_program(
+async def run_wandelscript_program(
     program_id: str,
     code: str,
-    args: dict[str, ElementType] = {},
+    parameters: dict[str, ElementType] = {},
     foreign_functions_paths: list[Path] | None = None,
     default_robot: str | None = None,
     default_tcp: str | None = None,
-) -> Program:
+    nova: Nova | None = None,
+) -> WandelscriptProgramRunner:
     logger.info(f"Creating wandelscript program: {program_id}")
     foreign_functions = (
         load_foreign_functions(foreign_functions_paths) if foreign_functions_paths else {}
     )
 
-    @nova.program(id=program_id)
-    async def wandelscript_wrapper():
-        async with Nova() as nova:
-            robot_cell = await nova.cell().get_robot_cell()
-            # This causes cyclic import, can't add to cell
-            cycle_device = CycleDevice(cell=nova.cell())
-            robot_cell.devices["cycle"] = cycle_device
+    if nova is None:
+        nova = Nova()
 
-            # TODO: Don't create another runner here, just execute the program
-            result = run(
-                program_id=program_id,
-                program=code,
-                args=args,
-                foreign_functions=foreign_functions,
-                robot_cell_override=robot_cell,
-                default_robot=default_robot,
-                default_tcp=default_tcp,
-            )
-            return result
+    async with nova:
+        cell = nova.cell()
+        controllers = await cell.controllers()
+        robot_cell = RobotCell(
+            timer=None, cycle=None, **{controller.id: controller for controller in controllers}
+        )
+        cycle_device = CycleDevice(cell=cell)
+        robot_cell.devices["cycle"] = cycle_device
 
-    return wandelscript_wrapper
+        runner = WandelscriptProgramRunner(
+            program_id=program_id,
+            code=code,
+            parameters=parameters,
+            default_robot=default_robot,
+            default_tcp=default_tcp,
+            foreign_functions=foreign_functions,
+            robot_cell_override=robot_cell,
+        )
+        runner.start(sync=True)
+        return runner
